@@ -1,6 +1,6 @@
 #
 #  This file is part of the Connection-Set Algebra (CSA).
-#  Copyright (C) 2010 Mikael Djurfeldt
+#  Copyright (C) 2010,2011,2012 Mikael Djurfeldt
 #
 #  CSA is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,9 +17,14 @@
 #
 
 from version import __version__
+from connset import Mask
+from connset import ConnectionSet
 from elementary import *
 #from operators import *
 #from arithmetic import *
 from misc import *
 from geometry import *
+from csaobject import parse, from_xml
 from plot import *
+from closure import *
+from conngen import *
