@@ -1,6 +1,6 @@
 #
 #  This file is part of the Connection-Set Algebra (CSA).
-#  Copyright (C) 2010 Mikael Djurfeldt
+#  Copyright (C) 2010,2011,2012 Mikael Djurfeldt
 #
 #  CSA is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@ import _misc
 def disc (r):
     return _misc.Disc (r)
 
+def rectangle (width, height):
+    return _misc.Rectangle (width, height)
+ 
 def gaussian (sigma, cutoff):
     return _misc.Gaussian (sigma, cutoff)
 
